/*
  Copyright (c) 2013-2014, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include "monitor_file.hpp"
#include "context.hpp"
#include "node.hpp"


namespace MMSS {

monitor_file_t::monitor_file_t(const std::string &filename) {
	file = std::fopen(filename.c_str(), "w");

	if (!file) {
		std::fprintf(stderr, "monitor_file: unable to open output file\n");
		std::exit(1);
	}
}

monitor_file_t::~monitor_file_t() {
	std::fclose(file);
}

void monitor_file_t::handle_log(node_t *node, int priority, const std::string &message) {
	std::fprintf(file, "%llu LOG %s %i %s\n", (unsigned long long)node->get_context()->now(), node->get_name().c_str(), priority, message.c_str());
}

void monitor_file_t::handle_init(node_t *node, const uint8_t *node_id, size_t len) {
	std::fprintf(file, "%llu INIT %s ", (unsigned long long)node->get_context()->now(), node->get_name().c_str());
	for (size_t i = 0; i < len; i++)
		std::fprintf(file, "%02x", node_id[i]);
	std::fprintf(file, "\n");
}

void monitor_file_t::handle_neigh(node_t *node, iface_t *iface, const gmrf_addr_t *addr, float rxcost, float txcost) {
	std::fprintf(file, "%llu NEIGH %s %s ", (unsigned long long)node->get_context()->now(), node->get_name().c_str(), iface->get_name().c_str());
	for (size_t i = 0; i < GMRF_ADDR_LEN; i++)
		std::fprintf(file, "%02x", addr->d[i]);
	std::fprintf(file, " %f %f\n", rxcost, txcost);
}

void monitor_file_t::handle_neigh_lost(node_t *node, iface_t *iface, const gmrf_addr_t *addr) {
	std::fprintf(file, "%llu NEIGH %s %s ", (unsigned long long)node->get_context()->now(), node->get_name().c_str(), iface->get_name().c_str());
	for (size_t i = 0; i < GMRF_ADDR_LEN; i++)
		std::fprintf(file, "%02x", addr->d[i]);
	std::fprintf(file, " LOST\n");
}

void monitor_file_t::handle_route(node_t *node, const uint8_t *node_id, size_t len, iface_t *iface, const gmrf_addr_t *addr, int metric) {
	std::fprintf(file, "%llu ROUTE %s ", (unsigned long long)node->get_context()->now(), node->get_name().c_str());
	for (size_t i = 0; i < len; i++)
		std::fprintf(file, "%02x", node_id[i]);
	std::fprintf(file, " %s ", iface->get_name().c_str());
	for (size_t i = 0; i < GMRF_ADDR_LEN; i++)
		std::fprintf(file, "%02x", addr->d[i]);
	std::fprintf(file, " %i\n", metric);
}

void monitor_file_t::handle_route_lost(node_t *node, const uint8_t *node_id, size_t len) {
	std::fprintf(file, "%llu ROUTE %s ", (unsigned long long)node->get_context()->now(), node->get_name().c_str());
	for (size_t i = 0; i < len; i++)
		std::fprintf(file, "%02x", node_id[i]);
	std::fprintf(file, " LOST\n");
}

}
